﻿using Plane.Copters;
using GalaSoft.MvvmLight;
using Microsoft.Practices.ServiceLocation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Plane.FormationCreator.Formation
{
    public partial class FlightTaskSingleCopterInfo : ObservableObject
    {
        public FlightTaskSingleCopterInfo(ICopter copter)
        {
            this.PropertyChanged += (sender, e) =>
            {
                switch (e.PropertyName)
                {
                    case nameof(TargetLat):
                    case nameof(TargetLng):
                    case nameof(TargetAlt):
                        _flightTaskManager.RaiseSingleCopterTaskChanged(this);
                        break;
                    default:
                        break;
                }
            };

            this.Copter = copter;
        }

        private CopterManager _copterManager = ServiceLocator.Current.GetInstance<CopterManager>();
        private FlightTaskManager _flightTaskManager = ServiceLocator.Current.GetInstance<FlightTaskManager>();

        public ICopter Copter { get; set; }

        private double? _TargetLat;
        public double TargetLat { get { return _TargetLat ?? Copter.Latitude; } set { Set(nameof(TargetLat), ref _TargetLat, value); } }

        private double? _TargetLng;
        public double TargetLng { get { return _TargetLng ?? Copter.Longitude; } set { Set(nameof(TargetLng), ref _TargetLng, value); } }

        private float? _TargetAlt;
        public float TargetAlt { get { return _TargetAlt ?? Copter.Altitude; } set { Set(nameof(TargetAlt), ref _TargetAlt, value); } }

        public LatLng LatLngOffset
        {
            get
            {
                var origin = GetOrigin();
                return new LatLng(TargetLat - origin.Lat, TargetLng - origin.Lng);
            }
            set
            {
                var origin = GetOrigin();
                TargetLat = origin.Lat + value.Lat;
                TargetLng = origin.Lng + value.Lng;
            }
        }

        public LatLng GetOrigin()
        {
            return _copterManager.Copters.GetCenter() ?? LatLng.Empty;
        }

        public bool isFinished;
        public int takeOffStage; // 判断起飞处于哪个阶段. (0 -> 1 ->2, 对应起飞到10m -> 水平飞行 -> 垂直飞行)
        public int RTLStage;     // 判断降落返航处于哪个阶段. (0->1->2, 对应降落到10m -> 水平飞行 -> 垂直降落)
        public int FlyToStage;   // 判断FlyTo处于哪个阶段
    }
}
