﻿using Plane.Copters;
using Plane.Geography;
using Plane.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Plane.FormationCreator.Formation
{
    public partial class FlightTask
    {
        public async Task RunCircleTaskAsync()
        {
            // 林俊清, 20150922, 首先一起调好方向。
            await Task.WhenAll(
                SingleCopterInfos.Select(info =>
                {
                    int copterIndex = SingleCopterInfos.IndexOf(info);
                    // 当该飞机被标记时，跳过飞行任务
                    if ((bool)_copterManager.CopterStatus[copterIndex])
                        return Task.Run(()=> { });
                    info.TargetHeading = info.CenterDirectionDeg;
                    return TurnTaskFlySingleCopterAsync(info);
                })
            ).ConfigureAwait(false);

            await Task.WhenAll(
                SingleCopterInfos.Select(info => CircleTaskFlySingleCopterAsync(info))
            ).ConfigureAwait(false);
        }

        private async Task CircleTaskFlySingleCopterAsync(FlightTaskSingleCopterInfo info)
        {
            // 林俊清, 20150907, 目前不对 Circle 进行仿真。
            if (info.Copter is FakeCopter)
            {
                await Task.Delay(1000);
                return;
            }

            int copterIndex = SingleCopterInfos.IndexOf(info);
            // 当该飞机被标记时，跳过飞行任务
            if ((bool)_copterManager.CopterStatus[copterIndex])
                return;

            await info.Copter.SetParamAsync("CIRCLE_RADIUS", info.Radius);

            await info.Copter.SetParamAsync("CIRCLE_RATE", info.Rate);

            var center = GeographyUtils.CalcLatLngSomeMetersAway2D(info.Copter.Latitude, info.Copter.Longitude, info.CenterDirectionDeg, 10);

            await info.Copter.CircleAsync();

            while (info.Copter.State != Copters.CopterState.Circling)
            {
                if (_flightTaskManager.IsPaused == true)
                {
                    await info.Copter.HoverAsync();
                    return;
                }
                await Task.Delay(25).ConfigureAwait(false);
            }

            await info.Copter.SetChannelsAsync(ch3: info.Channel3);

            for (int i = info.Turns; i > 0; i--)
            {
                //// 等待一段时间，让飞行器飞出一段距离。
                //for (int i = 0; i < 3000 / 25; i++)
                //{
                //    if (_flightTaskManager.IsPaused == true)
                //    {
                //        info.Copter.Hover();
                //        return;
                //    }
                //    await Task.Delay(25).ConfigureAwait(false);
                //}

                //var log = new StringBuilder(1024);
                //var lastLogTime = DateTime.Now;
                bool step1 = false;
                while (true)
                {
                    if (_flightTaskManager.IsPaused == true)
                    {
                        await info.Copter.HoverAsync();
                        return;
                    }
                    var angle = info.Radius == 0 ? NormalizeAngleDeg(info.Copter.Heading - info.CenterDirectionDeg) : GetNormalizedAngleDeg(info, center);
                    //log.AppendLine($"{angle} {step1}");
                    if (!step1)
                    {
                        if (angle > 100 && angle < 330)
                        {
                            step1 = true;
                            //log.AppendLine($"step1 is true now.");
                        }
                    }
                    else if (angle > 340 || angle < 5)
                    {
                        //log.AppendLine("break");
                        break;
                    }
                    //if (lastLogTime.AddSeconds(10) < DateTime.Now)
                    //{
                    //    Logger.Instance.Log(log.ToString());
                    //    log.Clear();
                    //    lastLogTime = DateTime.Now;
                    //}
                    await Task.Delay(25).ConfigureAwait(false);
                }
                //Logger.Instance.Log(log.ToString());
            }

            await info.Copter.HoverAsync();
        }

        private static double GetNormalizedAngleDeg(FlightTaskSingleCopterInfo info, Tuple<double, double> center)
        {
            var angle = info.Copter.InPlaneDirectionToDeg(center.Item1, center.Item2) - info.CenterDirectionDeg;
            return NormalizeAngleDeg(angle);
        }

        private static double NormalizeAngleDeg(double angle)
        {
            while (true)
            {
                if (angle < 0) angle += 360;
                else if (angle >= 360) angle -= 360;
                else break;
            }
            return angle;
        }
    }
}
