﻿using Plane.FormationCreator.Formation;
using Plane.FormationCreator.ViewModels;
using Plane.Logging;
using GalaSoft.MvvmLight.Ioc;
using Microsoft.Practices.ServiceLocation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Plane.FormationCreator
{
    public class ServiceLocatorConfigurer
    {
        public static ServiceLocatorConfigurer Instance { get; } = new ServiceLocatorConfigurer();

        private ServiceLocatorConfigurer()
        {
        }

        private SimpleIoc _container = SimpleIoc.Default;

        public void Configure()
        {
            ServiceLocator.SetLocatorProvider(() => _container);

            _container.Register<MainViewModel>();
            _container.Register<ConnectViewModel>();
            _container.Register<MapViewModel>();
            _container.Register<ControlPanelViewModel>();
            _container.Register<CopterListViewModel>();
            _container.Register<TaskBarViewModel>();
            _container.Register<ModifyTaskViewModel>();

            _container.Register<ILogger>(() => new LocalFileLogger(new DebugLogger()));

            _container.Register<CopterManager>();
            _container.Register<MapManager>();
            _container.Register<FlightTaskManager>();

            _container.Register<FormationController>();

            _container.Register<AppConfig>();
        }
    }
}
